db_dev_path = "test/db/dev.db"
db_path = "username:UR2hqJDbSfQ@username.mysql.pythonanywhere-services.com/username$default"
# db_path = "ENV:DBPATH"
db_mysql = True
sql_echo = False

use_alembic = True
migrations_folder = "test/alembic"

log_info_path = "test/logs/log_info.csv"
log_requests_path = "test/logs/log_requests.csv"
log_errors_path = "test/logs/log_errors.log"
log_frontend_path = "test/logs/log_frontend.log"

jwt_key_file_path = "test/secret_key_jwt.txt"

login_page_url = "/login"
api_url = "/api/"

blueprints_folder = "test/blueprints"
data_tables_folder = "test/data"
